#
# Awk program that takes as input a C++ header file and outputs
# a UNIX style man page.
#
# The contents of this file are public domain.
# Author: Bob Mastors
#         Epoch Systems, Inc.
#         2/90

BEGIN	{
	# specify how much to indent the various parts
	INDENT_1 = ""
	INDENT_2 = "  "
	INDENT_3 = "     "
	library= "UNKNOWN"
	arfilecount= 0
	}

/^#define/ {
	tmp= substr( $0, 8 )
	tmp= strip_white( tmp )
	gsub( "\\\\$", "", tmp );
	define = define INDENT_3 tmp "\n"
	}

/^#include/ {
	tmp= substr( $0, 9 )
	tmp= strip_white( tmp )
	include = include INDENT_3 tmp "\n"
	}

/^typedef/ {
	tmp= substr( $0, 8 )
	tmp= strip_white( tmp )
	typedef = typedef INDENT_3 tmp "\n"
	}

/^enum/ {
	tmp= substr( $0, 5 )
	gsub( "{", "", tmp );
	tmp= strip_white( tmp )
	enum= enum INDENT_2 tmp "\n"
	while( (getline tmp) > 0 )
		{
		gsub( "{", "", tmp );
		gsub( ",", " ", tmp );
		gsub( "//", "", tmp );
		tmp= strip_white( tmp )
		if( tmp ~ "} *;" )
			break;
		enum= enum INDENT_3 tmp "\n"
		}
	}

/^struct/ {
	tmp= substr( $0, 7 )
	tmp= strip_white( tmp )
	struct= struct INDENT_2 tmp "\n"
	while( (getline tmp) > 0 )
		{
		tmp= strip_white( tmp )
		gsub( "//", "  ", tmp );
		struct= struct INDENT_3 tmp "\n"
		if( tmp ~ "} *;" )
			break;
		}
	}

/^class/ {
	tmp= substr( $0, 6 )
	tmp= strip_white( tmp )
	gsub( "{", "", tmp )
	class= class doc
	doc= ""
	process_class( tmp )
	}

/^\/\// {
	tmp= substr( $0, 4 )
	striptmp= strip_white( tmp )
	if( gsub( "^\.FILE", "", striptmp ) == 1 )
		{
		striptmp= strip_white( striptmp )
		file= file INDENT_3 striptmp "\n"
		arfile[ striptmp ] = striptmp
		arfilecount++
		}
	else
	if( gsub( "^\.LIBRARY", "", striptmp ) == 1 )
		{
		striptmp= strip_white( striptmp )
		library= toupper( striptmp )
		}
	else
	if( gsub( "^\.SECTION", "", striptmp ) == 1 )
		{
		striptmp= strip_white( striptmp )
		striptmp= toupper( striptmp )
		doc= doc INDENT_1 striptmp "\n"
		doc_flag= "true"
		}
	else if( doc_flag == "true" )
		{
		doc= doc INDENT_2 tmp "\n"
		}
	}

NF == 0 {
	if( (doc_flag == "true") && (length(doc) > 0) )
		doc= doc "\n"
	doc_flag= ""
	}

END	{
	print
	tmp= toupper(FILENAME) "(c++) "
	printf( "%-35s %-10s %31s\n", tmp, library, tmp );
	"date '+%h %d, 19%y'" | getline tmp
	print
	printf( "%77s\n", tmp )
	print_section( "", class )
	print_section( "", doc )
	print_section( "DEFINED MACROS", define )
	print_section( "INCLUDED FILES", include )
	print_section( "TYPE DEFINITIONS", typedef )
	print_section( "ENUMERATIONS", enum )
	print_section( "STRUCTURES", struct )
	print_section( "SOURCE FILES", file )
	print_summarys()
	}

# prints out the specified header and data if the data string
# is not zero length
function print_section( header, data )
	{
	if( length(data) > 0 )
		{
		print INDENT_1 header
		print data
		}
	}

# Call to process a class.  Pass the name of the class.
function process_class( name,    inputstr,ii,bers )
	{
	# get lines of input until EOF
	ii= "private"
	while( (getline inputstr) > 0 )
		{
		# just get the token(s)
		inputstr= strip_white( inputstr )

		# ignore single {
		if( inputstr == "{" )
			continue

		# all done if found end of class definition
		if( inputstr ~ "} *;" )
			break;

		# public members follow
		if( inputstr ~ "public *:" )
			ii= "public"

		# private members follow
		else if( inputstr ~ "private *:" )
			ii= "private"

		# protected members follow
		else if( inputstr  ~ "protected *:" )
			ii= "protected"

		else if( (length(inputstr) > 0) || (length(bers[ii]) > 0) )
			{
			gsub( "//", "  ", inputstr )
			if( length(inputstr) > 0 )
				bers[ii]= bers[ii] INDENT_3 inputstr "\n"
			else
				bers[ii]= bers[ii] "\n"
			}
		}

	# construct the class string
	class= class INDENT_1 "CLASS " name "\n\n"
	if( length( bers[ "public" ] ) > 0 )
		{
		class= class INDENT_2 "Public members\n"
		class= class strip_nl( bers["public"] ) "\n\n"
		}
	if( length( bers[ "protected" ] ) > 0 )
		{
		class= class INDENT_2 "Protected members\n"
		class= class strip_nl( bers["protected"] ) "\n\n"
		}
	if( length( bers[ "private" ] ) > 0 )
		{
		class= class INDENT_2 "Private members\n"
		class= class strip_nl( bers["private"] ) "\n\n"
		}
	if( length( bers[ "friend" ] ) > 0 )
		{
		class= class INDENT_2 "Friends\n"
		class= class strip_nl( bers["friend"] ) "\n\n"
		}
	}

# strips leading and trailing whitespace from the specified string and
# returns the string
function strip_white( data )
	{
	gsub( "^[\t ]+", "", data )
	gsub( "[\t ]+$", "", data )
	return data
	}

# strips leading and trailing newlines from the specified string and
# returns the string
function strip_nl( data )
	{
	gsub( "\n", "@", data )
	gsub( "@+$", "", data )
	gsub( "@", "\n", data )
	return data
	}

# Call to get the function summarys from the source files and print them
function print_summarys(   data,title )
    {
    # just return if no source files listed
    if( arfilecount == 0 )
        return;

    # print the header
    print INDENT_1 "SUMMARY"

    # walk the list of source files
    for( xx in arfile )
	{
        # loop reading each line of the source file
	while( (getline inputstr <arfile[xx]) > 0 )
	    {
            # see if the start of the function summary
	    if( inputstr ~ "// Description:$" )
		{
		data= ""
		title= ""
                # loop reading the summary
		while( (getline inputstr <arfile[xx]) > 0 )
		    {
                    # see if all done
		    if( inputstr ~ "{.*" )
			{
			title= strip_nl( title )
			print title
			print data
			break;
			}

		    # strip the comment and leading whitespace
		    if( gsub( "//[\t ]*", "", inputstr ) != 0 )
			{
			# only keep lines not ending in :
			if( inputstr !~ ".*:$" )
			    {
                            # save the line of data
			    data= data INDENT_3 inputstr "\n"
			    }
			}
		    else
			{
			# only keep lines that are not blank
			if( length( inputstr ) )
			    {
                            # save the line of the function name
			    title= title INDENT_2 inputstr "\n"
			    }
			}
		    }
		}
	    }
	}
    }
